//+------------------------------------------------------------------+
//|                                                      Expert0.mqh |
//|                                      Copyright © 2017, Marketeer |
//|                           Portions of the code are public domain |
//|                          https://www.mql5.com/en/users/marketeer |
//|                                       Simple expert adviser stub |
//|                            https://www.mql5.com/ru/articles/3264 |
//+------------------------------------------------------------------+

class Double
{
  private:
    double v;
    double eps;
    void init()
    {
      eps = MathPow(10, -1 * _Digits);
    }
    
  public:
    Double(const double &d): v(d){init();}
    Double(double d): v(d){init();}
    Double(const Double &c): v(c.v){init();}
    
    double operator=(double x)
    {
      v = x;
      return v;
    }
    bool operator==(double x)
    {
      return MathAbs(v - x) < eps;
    }
    bool operator!=(double x)
    {
      return !(this == x);
    }
};

class Expert
{
  private:
    int magic;
    float lot;
    int deviation;
    string orderPrefix;
    datetime lastOrderTime;
  
  public:
    Expert(int m)
    {
      magic = m;
      lot = 0;
      deviation = 10;
    }

    Expert(int m, float l)
    {
      magic = m;
      lot = l;
      deviation = 10;
    }
    

    int getOrderCount()
    {
      return getOrderCount(-1);
    }

    int getMarketOrderCount()
    {
      return getOrderCount(mask(OP_BUY) | mask(OP_SELL));
    }

    int getOrderCount(const int op)
    {
      return getOrderCount(op, "");
    }
    
    int mask(int op)
    {
      return 1 << op;
    }

    /**
     op - mask of order types
    */
    int getOrderCount(const int op, const string CurrentSymbol)
    {
      int count = 0;
      for(int i = OrdersTotal() - 1; i >= 0; i--)
      {
        if(OrderSelect(i, SELECT_BY_POS))
        {
          int mask = 1 << OrderType();
          if(OrderMagicNumber() == magic && (CurrentSymbol == OrderSymbol() || CurrentSymbol == "") && ((mask & op) != 0))
          {
            count++;
          }
        }
      }
      return count;
    }

    void closeMarketOrders()
    {
      closeMarketOrders((1 << OP_BUY) | (1 << OP_SELL));
    }

    int closeMarketOrders(const int op)
    {
      return closeMarketOrders(op, Symbol());
    }

    int closeMarketOrders(const int op, const string CurrentSymbol)
    {
      int count = 0;
      for(int i = OrdersTotal() - 1; i >= 0; i--)
      {
        if(OrderSelect(i, SELECT_BY_POS))
        {
          int mask = 1 << OrderType();
          if(OrderMagicNumber() == magic && (CurrentSymbol == OrderSymbol() || CurrentSymbol == "") && ((mask & op) != 0))
          {
            double price = MarketInfo(OrderSymbol(), IsOfBuyType(OrderType())?MODE_BID:MODE_ASK);
            
            if(!OrderClose(OrderTicket(), OrderLots(), price, deviation))
            {
              Print("OrderClose failed for #", OrderTicket(), " ", OrderSymbol(), " with error ", GetLastError());
            }
            else
            {
              count++;
            }
          }
        }
      }
      return count;
    }

    void trailStops(const string CurrentSymbol, const int slbuy, const int slsell)
    {
      double point = MarketInfo(CurrentSymbol, MODE_POINT);
      
      for(int i = OrdersTotal() - 1; i >= 0; i--)
      {
        if(OrderSelect(i, SELECT_BY_POS))
        {
          if(OrderMagicNumber() == magic && (CurrentSymbol == OrderSymbol() || CurrentSymbol == "") && OrderType() < 2)
          {
            bool isBuy = IsOfBuyType(OrderType());
            double price = MarketInfo(OrderSymbol(), isBuy?MODE_BID:MODE_ASK);
            double level = price + (isBuy?-1:+1) * (isBuy?slbuy:slsell) * point;
            
            if(OrderStopLoss() == 0 || (isBuy && OrderStopLoss() < level) || (!isBuy && OrderStopLoss() > level))
            {
              if(MathAbs(OrderStopLoss() - level) / point >= deviation)
              {
                if(!OrderModify(OrderTicket(), OrderOpenPrice(), level, OrderTakeProfit(), OrderExpiration()))
                {
                  Print("OrderModify failed for #", OrderTicket(), " ", OrderSymbol(), " with error ", GetLastError());
                  Print(level, " ", Ask, " ", Bid);
                }
              }
            }
          }
        }
      }
    }
    
    void trailStops(const string CurrentSymbol, const int sl)
    {
      trailStops(CurrentSymbol, sl, sl);
    }

    int placeMarketOrder(const int type, const int sl, const int tp)
    {
      return placeMarketOrder(type, Symbol(), sl, tp);
    }

    int placeMarketOrder(const int type)
    {
      return placeMarketOrder(type, Symbol(), 0, 0);
    }
    
    double volumeCalculatePercent(const string symbol, const double percent, double customMargin = 0)
    {
      double NewLots = 0;
      double OneLot = MarketInfo(symbol, MODE_MARGINREQUIRED);
      if(OneLot == 0)
      {
        Print("Margin is zero ", symbol);
        return(0);
      }
      double MinLot = MarketInfo(symbol, MODE_MINLOT);
      if(MinLot == 0)
      {
        Print("MinLot is zero ", symbol);
        return(0);
      }
      double Step = MarketInfo(symbol, MODE_LOTSTEP);
      if(Step == 0)
      {
        Print("LotStep is zero");
        return(0);
      }
    
      double Money;
      if(customMargin == 0)
      {
        Money = AccountFreeMargin();
      }
      else
      {
        Money = customMargin;
      }
    
      NewLots = Money * MathMin(percent, 100) / 100 / OneLot;
      
      NewLots = MathFloor(NewLots / Step) * Step;
    
      if(NewLots < MinLot)
      {
        return 0;
      }
      
      double MaxLot = MarketInfo(symbol, MODE_MAXLOT);
      if(NewLots > MaxLot) NewLots = MaxLot;
      
      return(NewLots);
    }
    
    int placeMarketOrder(const int type, const string CurrentSymbol, const int sl, const int tp)
    {
      double point = MarketInfo(CurrentSymbol, MODE_POINT);
      double l = lot;
      if(l == 0) l = MarketInfo(CurrentSymbol, MODE_MINLOT);
      if(l < 0) l = volumeCalculatePercent(CurrentSymbol, -l);
      
      double price, stoploss, takeprofit;
      if(IsOfBuyType(type))
      {
        price = MarketInfo(CurrentSymbol, MODE_ASK);
        stoploss = sl > 0 ? price - sl * point : 0;
        takeprofit = tp > 0 ? price + tp * point : 0;
      }
      else
      {
        price = MarketInfo(CurrentSymbol, MODE_BID);
        stoploss = sl > 0 ? price + sl * point : 0;
        takeprofit = tp > 0 ? price - tp * point : 0;
      }
      
      int ticket = (int)OrderSend(CurrentSymbol, type, l, price, deviation, stoploss, takeprofit, orderPrefix, magic);
      if(ticket == -1)
      {
        Print("OrderSend failed ", CurrentSymbol, type, l, price, stoploss, takeprofit, " with error ", GetLastError());
      }
      else
      {
        lastOrderTime = iTime(CurrentSymbol, 0, 0);
      }
      return ticket;
    }
    
    int getLastOrderBar()
    {
      return iBarShift(Symbol(), 0, lastOrderTime, true);
    }
    
    bool orderModify(const int ticket, const double price, const double sl, const double tp, const datetime exp)
    {
      if(!OrderSelect(ticket, SELECT_BY_TICKET))
      {
        return false;
      }
      Double oop(OrderOpenPrice()), osl(OrderStopLoss()), otp(OrderTakeProfit());
      if(oop != price || osl != sl || otp != tp || OrderExpiration() != exp)
      {
        if(MathAbs(sl - price) / Point > MarketInfo(OrderSymbol(), MODE_SPREAD) + MarketInfo(OrderSymbol(), MODE_STOPLEVEL))
        {
          bool result = OrderModify(ticket, price, sl, tp, exp);
          if(!result)
          {
            Print("OrderModify ", OrderType(), " failed for #", OrderTicket(), " ", OrderSymbol(), " with error ", GetLastError());
            Print("Ask ", Ask, " Bid ", Bid);
            Print("OOP P:", OrderOpenPrice(), " -> ", price, " SL:", OrderStopLoss(), " -> ", sl, " TP:", OrderTakeProfit(), " -> ", tp);
          }
          return result;
        }
        else
        {
          Print("Too close, skip");
        }
      }
      return false;
    }
    

    bool IsOfBuyType(int op)
    {
      return(!(op % 2));
    }

};
