//+------------------------------------------------------------------+
//|                                                   MarketMQL4.mq5 |
//|                                      Copyright © 2010, Marketeer |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+

#define MODE_LOW 1
#define MODE_HIGH 2
#define MODE_TIME 5
#define MODE_BID 9
#define MODE_ASK 10
#define MODE_POINT 11
#define MODE_DIGITS 12
#define MODE_SPREAD 13
#define MODE_STOPLEVEL 14
#define MODE_LOTSIZE 15
#define MODE_TICKVALUE 16
#define MODE_TICKSIZE 17
#define MODE_SWAPLONG 18
#define MODE_SWAPSHORT 19
#define MODE_STARTING 20
#define MODE_EXPIRATION 21
#define MODE_TRADEALLOWED 22
#define MODE_MINLOT 23
#define MODE_LOTSTEP 24
#define MODE_MAXLOT 25
#define MODE_SWAPTYPE 26
#define MODE_PROFITCALCMODE 27
#define MODE_MARGINCALCMODE 28
#define MODE_MARGININIT 29
#define MODE_MARGINMAINTENANCE 30
#define MODE_MARGINHEDGED 31
#define MODE_MARGINREQUIRED 32
#define MODE_FREEZELEVEL 33

double MarketInfo(string symbol, int type)
{
  //MqlTick last_tick;
  double Ask, Bid;
  //bool Success;
  
  switch(type)
  {
    case MODE_LOW:
       return(SymbolInfoDouble(symbol,SYMBOL_LASTLOW));
    case MODE_HIGH:
       return(SymbolInfoDouble(symbol,SYMBOL_LASTHIGH));
    case MODE_TIME:
       return((double)SymbolInfoInteger(symbol,SYMBOL_TIME));
    case MODE_BID:
       //Success = SymbolInfoTick(symbol,last_tick);
       Bid = SymbolInfoDouble(symbol, SYMBOL_BID);
       return(Bid);
    case MODE_ASK:
       //Success = SymbolInfoTick(symbol,last_tick);
       Ask = SymbolInfoDouble(symbol, SYMBOL_ASK);
       return(Ask);
    case MODE_POINT:
       return(SymbolInfoDouble(symbol,SYMBOL_POINT));
    case MODE_DIGITS:
       return((double)SymbolInfoInteger(symbol,SYMBOL_DIGITS));
    case MODE_SPREAD:
       return((double)SymbolInfoInteger(symbol,SYMBOL_SPREAD));
    case MODE_STOPLEVEL:
       return((double)SymbolInfoInteger(symbol,SYMBOL_TRADE_STOPS_LEVEL));
    case MODE_LOTSIZE:
       return(SymbolInfoDouble(symbol,SYMBOL_TRADE_CONTRACT_SIZE));
    case MODE_TICKVALUE:
       return(SymbolInfoDouble(symbol,SYMBOL_TRADE_TICK_VALUE));
    case MODE_TICKSIZE:
       return(SymbolInfoDouble(symbol,SYMBOL_TRADE_TICK_SIZE));
    case MODE_SWAPLONG:
       return(SymbolInfoDouble(symbol,SYMBOL_SWAP_LONG));
    case MODE_SWAPSHORT:
       return(SymbolInfoDouble(symbol,SYMBOL_SWAP_SHORT));
    case MODE_STARTING:
       return((double)SymbolInfoInteger(symbol,SYMBOL_START_TIME));
    case MODE_EXPIRATION:
       return((double)SymbolInfoInteger(symbol,SYMBOL_EXPIRATION_TIME));
    case MODE_TRADEALLOWED:
       return(SymbolInfoInteger(symbol,SYMBOL_TRADE_MODE) == SYMBOL_TRADE_MODE_DISABLED);
    case MODE_MINLOT:
       return(SymbolInfoDouble(symbol,SYMBOL_VOLUME_MIN));
    case MODE_LOTSTEP:
       return(SymbolInfoDouble(symbol,SYMBOL_VOLUME_STEP));
    case MODE_MAXLOT:
       return(SymbolInfoDouble(symbol,SYMBOL_VOLUME_MAX));
    case MODE_SWAPTYPE:
       return((double)SymbolInfoInteger(symbol,SYMBOL_SWAP_MODE));
    case MODE_PROFITCALCMODE:
       return((double)SymbolInfoInteger(symbol,SYMBOL_TRADE_CALC_MODE));
    case MODE_MARGINCALCMODE:
       return(0);
    case MODE_MARGININIT: 
       return(SymbolInfoDouble(symbol,SYMBOL_MARGIN_INITIAL));
    case MODE_MARGINMAINTENANCE:
       return(SymbolInfoDouble(symbol,SYMBOL_MARGIN_MAINTENANCE));
    case MODE_MARGINHEDGED:
       return(0);
    case MODE_MARGINREQUIRED:
       return(MarginCalculate(symbol, 1.0));
    case MODE_FREEZELEVEL:
       return((double)SymbolInfoInteger(symbol,SYMBOL_TRADE_FREEZE_LEVEL));
  
    default: return(0);
  }
  return(0);
}

int AccountLeverage()
{
  return((int)AccountInfoInteger(ACCOUNT_LEVERAGE));
}

double AccountFreeMargin()
{
  return(AccountInfoDouble(ACCOUNT_FREEMARGIN));
}

double AccountFreeMarginCheck(string symbol, int cmd, double volume)
{
  double FreeMargin = AccountFreeMargin() - volume * MarketInfo(symbol, MODE_MARGINREQUIRED);
  return(MathMax(0, FreeMargin));
}

string AccountCurrency()
{
  return(AccountInfoString(ACCOUNT_CURRENCY));
}

double AccountBalance()
{
  return(AccountInfoDouble(ACCOUNT_BALANCE));
}

double AccountEquity()
{
  return(AccountInfoDouble(ACCOUNT_EQUITY));
}

double MarginCalculate(string symbol, double volume)
{
  string first    = StringSubstr(symbol, 0, 3);        // первый символ,    например EUR
  string second   = StringSubstr(symbol, 3, 3);        // второй символ,    например USD
  string currency = AccountCurrency();                 // валюта депозита,  например USD
  //Print("accountcur=", currency);
  if(currency == "") currency = "USD";
  double leverage = AccountLeverage();                 // кредитное плечо,  например 100
  //Print("accountlev=", leverage);
  if(leverage == 0) leverage = 500;
  double contract = MarketInfo(symbol, MODE_LOTSIZE);  // размер контракта, например 100000
  double bid      = MarketInfo(symbol, MODE_BID);      // цена бид


  // допускаем только стандартные форексные символы XXXYYY
  //if(StringLen(symbol) != 6)
  //{
  //  Print("MarginCalculate: \'", symbol ,"\' is not a standard forex symbol XXXYYY");
  //  return(GetDefaultMargin(symbol, volume));
  //}
  
  // проверка наличия данных
  if(bid <= 0 || contract <= 0) 
  {
    Print("MarginCalculate: no market information for \'", symbol, "\'");
    return(0.0);
  }
  // проверяем самые простые варианты - без кроссов
  if(first == currency)  return(contract*volume/leverage);           // USDxxx
  if(second == currency) return(contract*bid*volume/leverage);       // xxxUSD
  // проверяем обычные кроссы, ищем прямое преобразование через валюту депозита
  string base = currency + first;                                    // USDxxx
  //Print("base=", base, " ", MarketInfo(base, MODE_BID));
  if(MarketInfo(base, MODE_BID) > 0) return(contract/MarketInfo(base,MODE_BID)*volume/leverage);
  // попробуем наоборот
  base = first + currency;                                           // xxxUSD
  //Print("base=", base, " ", MarketInfo(base, MODE_BID), " contract=", contract);
  if(MarketInfo(base, MODE_BID) > 0) return(contract*MarketInfo(base,MODE_BID)*volume/leverage);
  // нет возможности прямого пересчета
  
  //double x = GetDefaultMargin(symbol, volume);
  
  Print("MarginCalculate: can not convert \'", symbol, "\' default method gives 0");
  
  return(0);
}
